--[[
Class: UDPROXY
Simplifies calling <UDPIPE>, by serializing command names and its arguments, and sending it to the server.
It also wraps the server responses, calling callbacks function with unserialized arguments.
It uses http://www-users.rwth-aachen.de/David.Kolf/json-lua for JSON.

Warning: it is still a work-in-progress, yet it seems to be already useful. For now it is used only for the client side.

Note: you should use the instance of UDPIPE (created my UDPIPE()), not the instance of subclassed class. As this is the proxy object, it overrides the __index metamethod, so OO will not work. 

Usage:
This should simplify writing client side code. But it requires some changes on the server side, notably the data should be serialized using json as "command=<JSON Serialized arguments>"

Client:
(start code)
UDPIPE=require 'UDPIPE'
UDPROXY=require 'UDPROXY'
conn=UDPIPE()
conn:connect('127.0.0.1', 1110)

Logic=UDPROXY(conn)

function Logic:onReply(M)
  print('Reply count:'..M.counter..' content:'..M.msg)
end

function conn:onConnect()
  Logic:login('player1', 'secretpasswd')
  Logic:getGameState()
end

while true do
  Logic:update()
end  
(end)

Server code:
It is just a simple echo server.
(start code)
UDPIPE=require 'UDPIPE'
json=require 'dkjson'
server=UDPIPE()

host, port='127.0.0.1', 1110

server:start(port, host)

function server:onNewMessage(msg, sock)
  counter=(counter or 0)+1
  self:send('Reply='..json.encode({msg=msg, counter=counter}), sock)
end

while true do
  server:step()
end
(end)

--]]
local M={}
local mt={}
local Connection=nil
local Logic=nil
local json=require 'dkjson'

local print=function(...) end

setmetatable(M, mt)

function mt.__index(a, cmd)
    return function(self, ...)
      Connection:send(cmd..'='..json.encode({...}))
    end
end

function mt.__tostring(self)
  local n=rawget(self, '_name') or 'UnNamed'
  return '<UDPROXY '..n..'>'
end

local function update(self, dt)
  Connection:step(dt)
end

function mt.__call(self, conn, l)
  Connection=conn
  Logic=l
  local obj={_name='Obj-'..math.random(999), update=update}
  
  function Connection.onNewMessage(conn, msg, sock)
    local cmd, param=msg:match('(%w+)=(.+)')
    if cmd then
      --print('GOT cmd, param:', cmd, param)
      cmd='on'..cmd
      local func, self2
      if Logic then      
        func=Logic[cmd]
        self2=Logic
      else
        func=rawget(obj, cmd)
        self2=obj
      end
      if func then
        --print('Calling:', obj, cmd, param)
        local dparam, _=json.decode(param)
        func(self2, dparam)
      else
        print('Command '..cmd..' not defined!')
      end
    else
      print('Got unknown mesage:', msg)
    end
  end
  return setmetatable(obj, mt)
end
return M
