local lg=love.graphics
local UDPIPE=require 'UDPIPE'
local json=require 'dkjson'
local GameLogic=require 'GameLogic'
username=os.getenv('USER') or 'guest'
password='secret'

local print=function(...) end

MAXW, MAXH=1024,768
buf=''

function love.load()
  lg.setMode(MAXW, MAXH)
  Game=GameLogic()
  Game:setSize(MAXW, MAXH)
  Game:initialize()
  Game:login(username, password)
  GameState=Game:getGameState()
end

function love.draw()
  lg.setLineWidth(2)
  
  for k,v in ipairs(GameState.Walls) do
    lg.setColor(0,255,0)
    lg.rectangle('fill', unpack(v))
    lg.setColor(0,0,255)
    lg.rectangle('line', unpack(v))
  end
  
  for k,v in pairs(GameState.Players) do
    if v.color then
      lg.setColor(unpack(v.color))
    else
      lg.setColor(255,0,0)
    end
    lg.circle('fill', v.x, v.y, 15)
    lg.print(v.name, v.x+10, v.y-30)
  end

  lg.setColor(255,255,255)
  for k,v in ipairs(GameState.Msgs) do
    lg.print(v[1]..': '..v[2], 0, k*14)
  end
  --[[
  if lastmsg then
    lg.print(lastmsg, 0, MAXH-20)
  end
  --]]
  --[[
  if status then
    lg.print(status, 0, MAXH-40)
  end
  --]]
    --lg.print(username..' '..PKTS..'/'..Client:getSent(), MAXW-120, 10)
    lg.print(username, MAXW-120, 10)
--[[
  local p=G.Players[username]
  if p then
    lg.print('X='..p.x..' Y='..p.y, MAXW-120, 20)
  end
  --]]
  --[[
  lg.translate(100,50)
  lg.rotate(ROT)
  lg.translate(-100,-50)
  lg.line(100,50,120,50)
  --]]
end


function love.keypressed(k, u)
  if k=='q' or k=='escape' then
    Game:logout(username)
    love.event.push('q')
    return
  end
  
  if k=='f1' then
    Game:initialize()
    GameState=Game:getGameState()
    return
  end
  if k=='f2' then
    for k,v in pairs(GameState.Players) do print(k,v) end
    for k,v in pairs(GameState.Msgs) do print(k,v[1], v[2]) end
    return
  end
  if k=='f3' then
    local numPlayers=0
    for k,v in pairs(GameState.Players) do
      numPlayers=numPlayers+1
    end
    Game:addPlayer('player'..(numPlayers+1))
    return
  end
  if k=='return' then
    print('Buffer: ', buf)
    --G:say('player1', buf)
    Game:say(buf)
    buf=''
  elseif #k==1 then
    buf=buf..k
  end
end

local isDown=love.keyboard.isDown
function love.update(dt)
  DT=(DT or 0)+dt
  if DT<0.05 then
    return
  end
  ----Client:step(dt)  
  --Client:step(DT)
  Game:update(dt)
  GameState=Game:getGameState()
  DT=DT-0.05

  local d=1
  if isDown('lshift') or isDown('rshift') then
    d=3
  end

  local dx,dy=0,0

  local p=GameState.Players[username]
  if not p then 
    status='NO PLAYER!'
    return
  end

  if isDown('right') then
    dx=d
  elseif(isDown('left')) then
    dx=-d
  end
  if isDown('up') then
    dy=-d
  elseif isDown('down') then
    dy=d
  end
  if dx~=0 or dy~=0 then
    local nx,ny=p.x+dx, p.y+dy
    if NEXTX~=nx or NEXTY~=ny then
      --Client:send('MOVETO X='..nx..' Y='..ny)
      Game:movePlayerTo(username, nx, ny)
      NEXTX, NEXTY=nx, ny
    end
  end
end
