require 'class'

local Player=require 'Player'

MAXW, MAXH=1024,768

local rand=math.random
math.randomseed(os.time())

local M=class(function(self)
  self:initialize()
end)

function M:getSize()
  return MAXW, MAXH
end

function M:setSize(w, h)
  MAXW=w
  MAXH=h
  return self
end

function M:updateState(msg, data)
  if data then
    print(msg, data)
  else
    print('GameState updated: '..msg)
  end
end

function M:initialize()
  self.Players={}
  self.Walls={}
  self.Msgs={}

  local W={}
  for i=1,20 do
    local x,y=rand(MAXW), rand(MAXH)
    local w,h=10+rand(150), 10+rand(150)
    W[i]={x,y,w,h}
  end
  self.Walls=W
  self:say('system', 'Game initialized!')
  return self
end

function M:checkWallCollision(X, Y, delta)
  delta=delta or 0
  local Collisions={}
  for k,v in ipairs(self.Walls) do
    local x,y,w,h=unpack(v)
    if X+delta>=x and X-delta<=x+w and Y+delta>=y and Y-delta<=y+h then
      Collisions[#Collisions+1]=v
    end
  end
  if #Collisions>0 then
    return Collisions
  end
end

function M:checkPlayerCollision(X, Y, delta, skip)
  delta=delta or 15
  local Collisions={}
  for k,v in pairs(self.Players) do
    local x,y=v.x,v.y
    if v.name~=skip and X+delta>=x and X-delta<=x and Y+delta>=y and Y-delta<=y then
      Collisions[#Collisions+1]=v
    end
  end
  if #Collisions>0 then
    return Collisions
  end
end

function M:login(user, password)
  self:addPlayer(user)
end

function M:logout(user)
  self:removePlayer(user)
end

function M:addNewPlayer()
  local n=0
  for k,v in pairs(self.Players) do
    n=n+1
  end
  self:addPlayer('player_'..n)
end

function M:addPlayer(name, color)
  if self.Players[name] then
    print('Did not add already existing player '..name)
    return
  end
  local p=Player(name, color)
  p:setPosition(rand(MAXW), rand(MAXH))
  while self:checkWallCollision(p.x, p.y, 15) or self:checkPlayerCollision(p.x, p.y, 15*2, p.name) do
    p.x, p.y=rand(MAXW), rand(MAXH)
  end
  self.Players[name]=p
  self:updateState('AddPlayer', p)
  return self
end

function M:removePlayer(name)
  if self.Players[name] then
    self.Players[name]=nil
    self:updateState('RemovePlayer', name)
  end
  return self
end

function M:movePlayerTo(playername, x, y)
  local p=self.Players[playername]
  if not p then return end
  if x<15 then x=15 end
  if x>MAXW-15 then x=MAXW-15 end
  if y<15 then y=15 end
  if y>MAXH-15 then y=MAXH-15 end
  local px,py=p.x, p.y
  local coll=self:checkWallCollision(x, y, 15)
  if coll then
    local msg='Ops, there is a wall'
    self:say(playername, msg)
    return
  end
  local dx,dy=x-px, y-py
  coll=self:checkPlayerCollision(x, y, 15*2, playername)
  if coll then
    for k,v in ipairs(coll) do
      local msg='Ouch! You hit me, '..playername..'!'
      self:say(v.name, msg)
    end
    return
  end
  p.x,p.y=x,y
  self:updateState('PlayerUpdated', {name=playername, x=p.x, y=p.y})
  return self
end


function M:getPlayers()
  return self.Players
end

function M:getGameState()
  local S={Walls=self.Walls, Players=self.Players, Msgs=self.Msgs}
  return S
end

function M:update(dt)
  -- player auto changes position every 0.5 sec (if it exists)
  local p=self.Players['auto']
  if p then
    CUM=(CUM or 0) + dt
    if CUM>=0.5 then
      CUM=CUM-0.5
      local x,y=p.x+rand(-4,4), p.y+rand(-4,4)
      self:movePlayerTo(p.name, x, y)
    end
  end
end

function M:say(playername, msg)
  playername=playername or 'unnamed'
  msg=msg or '[no message]'
  local lastMsg=self.Msgs[#self.Msgs]
  if lastMsg and lastMsg[1]==playername and lastMsg[2]==msg then return end -- don't send the same message twice in a row
  local M={playername, msg}
  self.Msgs[#self.Msgs+1]=M
  self:updateState('Say', {name=playername, msg=msg})
  -- Keep only last 5 messages
  if #self.Msgs>5 then
    table.remove(self.Msgs, 1)
  end
end

function M:__tostring()
  return '<GameLogic>'
end

return M
