#!/usr/bin/lua

--UDPIPE=require 'UDPIPE'
UDPIPE=require 'UDPIPEVERBOSE'
GameLogic=require 'GameLogic'
local json=require 'dkjson'

local print=function(...) end

Game=GameLogic()
Game:initialize()

ServerConnection=class(UDPIPE, function(self, ...)
  UDPIPE.init(self, ...)
end)

local gettime=socket.gettime
-- Main server loop
function ServerConnection:run()
  self:start(1111, '127.0.0.1')
  local t1, t2=gettime()
  while true do
    t2=gettime()
    self:step(t2-t1)
    Game:update(t2-t1) -- update game logic
    socket.sleep(0.001) -- dont overload the CPU
    t1=t2
  end
end


function ServerConnection:onNewMessage(msg, sock)
  local cmd, param=msg:match('(%w+)=(.+)')
  if not cmd then cmd=msg end
  local func=Game[cmd]
  if func and type(func)=='function' then
    local res
    if param then
      local p1, _=json.decode(param)
      res={func(Game, unpack(p1))}
    else
      res={func(Game)}
    end
    if next(res) then
      local k,v=next(res)
      if v~=Game then
        self:send(cmd..'='..json.encode(unpack(res)), sock) -- serialize results and send an answer
      else
        print('Skipping...') -- Game returns "self", so nothing interesting to send out
      end
    end
  else
    print('Unknown command: '..cmd, sock)
    self:send('error='..json.encode({msg='Unknown command: '..cmd}), sock)
  end
end

server=ServerConnection()

function Game:updateState(msg, data)
  if data then
    server:sendAll(msg..'='..json.encode(data))
  end
end

server:run()
