--[[
Class: UDPROXY
Simplifies calling UDPIPE, by serializing command names and its arguments, and sending it to the server.
It also wraps the server responses, calling callbacks function with unserialized arguments.

Usage:
(start code)
conn=UDPIPE()
Logic=UDPROXY(conn)

function Logic:onLoggedIn(username, msg)
  print('User '..username..' logged in: '..msg)
end

Logic:login('player1', 'secretpasswd')

while true do
  Logic:update()
end  
(end)
--]]
local M={}
local mt={}
local Connection=nil
local Logic=nil
local json=require 'dkjson'

local print=function(...) end

setmetatable(M, mt)

function mt.__index(a, cmd)
    return function(self, ...)
      Connection:send(cmd..'='..json.encode({...}))
    end
end

function mt.__tostring(self)
  local n=rawget(self, '_name') or 'UnNamed'
  return '<UDPROXY '..n..'>'
end

local function update(self, dt)
  Connection:step(dt)
end

function mt.__call(self, conn, l)
  Connection=conn
  Logic=l
  local obj={_name='Obj-'..math.random(999), update=update}
  
  function Connection.onNewMessage(conn, msg, sock)
    local cmd, param=msg:match('(%w+)=(.+)')
    if cmd then
      --print('GOT cmd, param:', cmd, param)
      cmd='on'..cmd
      local func, self2
      if Logic then      
        func=Logic[cmd]
        self2=Logic
      else
        func=rawget(obj, cmd)
        self2=obj
      end
      if func then
        --print('Calling:', obj, cmd, param)
        local dparam, _=json.decode(param)
        func(self2, dparam)
      else
        print('Command '..cmd..' not defined!')
      end
    else
      print('Got unknown mesage:', msg)
    end
  end
  return setmetatable(obj, mt)
end
return M
