local lg=love.graphics
local Game=require 'ClientGameLogic'
username=os.getenv('USER') or 'guest'
password='secret'

---local print=function(...) end

MAXW, MAXH=1024,768
buf=''

function love.load()
  lg.setMode(MAXW, MAXH)
  Game:setUser(username, password)
  FB=lg.newFramebuffer()
end

function love.draw()
  if CACHING then
    if not CACHED then
      pcall(FB.clear, FB)  -- for love 0.8
      lg.setRenderTarget(FB)
    end -- CACHED
  end -- CACHING

  if not CACHING or not CACHED then

  lg.setLineWidth(2)

  -- Draw walls
  for k,v in ipairs(GameState.Walls) do
    lg.setColor(0,255,0)
    lg.rectangle('fill', unpack(v))
    lg.setColor(0,0,255)
    lg.rectangle('line', unpack(v))
  end
  
  -- Draw players
  for k,v in pairs(GameState.Players) do
    if v.color then
      lg.setColor(unpack(v.color))
    else
      lg.setColor(255,0,0)
    end
    lg.circle('fill', v.x, v.y, 15)
    lg.print(v.name, v.x+10, v.y-30)
  end

  -- Draw messages
  lg.setColor(255,255,255)
  for k,v in ipairs(GameState.Msgs) do
    lg.print(v[1]..': '..v[2], 0, k*14)
  end

  -- Draw player name
  lg.print(username, MAXW-120, 10)
  lg.print(string.format('Pkts: %d/%d', Game:getSentPackets(), Game:getReceivedPackets()), MAXW-320, 60)
  end

  local wascached=CACHED

  if CACHING then
    lg.setColor(255,255,255)
    if not CACHED then
      lg.setRenderTarget()
      CACHED=true
    end -- CACHED
    lg.draw(FB, 0, 0, 0)
    lg.print('CACHING', MAXW-120, 80)
  else
    lg.print('NOT CACHING', MAXW-120, 80)
  end -- CACHING
end

function love.keypressed(k, u)
  if k=='q' or k=='escape' then
    Game:logout(username)
    love.event.push('q')
    return
  end
 
  -- F1 - reinitialize the game
  if k=='f1' then
    Game:initialize()
    GameState=Game:getGameState(true)
    CACHED=nil
    return
  end
  -- F2 - add a self-moving player
  if k=='f2' and not GameState.Players.auto then
    Game:addPlayer('auto')
    CACHED=nil
    return
  end
  -- F3 - add another player
  if k=='f3' then
    Game:addNewPlayer()
    CACHED=nil
    return
  end
  -- F4 - get the current state`
  if k=='f4' then
    GameState=Game:getGameState(true)
    CACHED=nil
    return
  end
  if k=='f5' then
    Game:login(username, password)
    GameState=Game:getGameState(true)
    CACHED=nil
  end
  if k=='f6' then
    Game:removePlayer('auto')
    CACHED=nil
  end
  if k=='f7' then
    CACHING=not CACHING
  end
  -- Send a message...
  if k=='return' then
    Game:say(username, buf)
    buf=''
    CACHED=nil
  elseif #k==1 then
  -- ...or keep writing it
    buf=buf..k
  end
end

local isDown=love.keyboard.isDown
function love.update(dt)
  DT=(DT or 0)+dt
  if DT>0.005 then  -- don't update game more often than that
    Game:update(math.max(DT, 0.005))
    DT=DT-math.max(DT, 0.005)
    CACHED=nil
  end
  GameState=Game:getGameState()

  local d=1
  -- Move faster with SHIFT
  if isDown('lshift') or isDown('rshift') then
    d=3
  end

  local dx,dy=0,0 -- new position

  local p=GameState.Players[username]
  if not p then 
    --error('No player!')
    return
  end

  if isDown('right') then
    dx=d
  elseif(isDown('left')) then
    dx=-d
  end
  if isDown('up') then
    dy=-d
  elseif isDown('down') then
    dy=d
  end
  if dx~=0 or dy~=0 then
    if LASTMOVE then
      LASTMOVE=LASTMOVE+dt
      if LASTMOVE>0.05 then
        LASTMOVE=nil -- reset last move timer. See also: ClientGameLogic:onPlayerUpdated
      end
    end
    if not LASTMOVE then
      local nx,ny=p.x+dx, p.y+dy
      Game:movePlayerTo(username, nx, ny)
      CACHED=nil
      LASTMOVE=0 -- start last move timer
    else
      -- skip moving - don't do it too often
    end
  end
end
